@@ -88,11 +88,6 @@ func formatWalker(dir *fs.Dir, maxDepth int) error {
 func formatDir(dir *fs.Dir) error {
 	var totalReformatCount int
 
-	ignoreTable, err := dir.Config.GetRegexp("ignore-table")
-	if err != nil {
-		return NewExitValue(CodeBadConfig, err.Error())
-	}
-
 	// Get workspace options for dir. This involves connecting to the first defined
 	// instance, so that any auto-detect-related settings work properly. However,
 	// with workspace=docker we can ignore connection errors; we'll get reasonable
@@ -104,7 +99,7 @@ func formatDir(dir *fs.Dir) error {
 			if err != nil {
 				return NewExitValue(CodeBadConfig, err.Error())
 			} else if inst == nil {
-				return NewExitValue(CodeBadConfig, "With workspace=temp-schema, this command needs a host to operate on, but no host is defined for environment %q", dir.Config.Get("environment"))
+				return NewExitValue(CodeBadConfig, "This command needs either a host (with workspace=temp-schema) or flavor (with workspace=docker), but one is not configured for environment %q", dir.Config.Get("environment"))
 			}
 		}
 		if wsOpts, err = workspace.OptionsForDir(dir, inst); err != nil {
@@ -125,7 +120,6 @@ func formatDir(dir *fs.Dir) error {
 
 		dumpOpts := dumper.Options{
 			IncludeAutoInc: true,
-			IgnoreTable:    ignoreTable,
 			CountOnly:      !dir.Config.GetBool("write"),
 		}
 		if dir.Config.GetBool("strip-partitioning") {
@@ -139,7 +133,7 @@ func formatDir(dir *fs.Dir) error {
 		totalReformatCount += reformatCount
 	}
 
-	for _, stmt := range dir.IgnoredStatements {
+	for _, stmt := range dir.UnparsedStatements {
 		log.Debugf("%s: unable to parse statement", stmt.Location())
 	}
 	if totalReformatCount > 0 {
