@@ -98,6 +98,7 @@ func InitHandler(cfg *mybase.Config) error {
 
 	// Iterate over the schemas. For each one, create a dir with .skeema and *.sql files
 	for _, s := range schemas {
+		s.StripMatches(hostDir.IgnorePatterns)
 		if err := PopulateSchemaDir(s, hostDir, separateSchemaSubdir); err != nil {
 			return err
 		}
@@ -152,7 +153,7 @@ func createHostOptionFile(cfg *mybase.Config, hostDir *fs.Dir, inst *tengo.Insta
 	if flavor := inst.Flavor(); flavor.Known() {
 		hostOptionFile.SetOptionValue(environment, "flavor", flavor.Family().String())
 	}
-	for _, persistOpt := range []string{"user", "ignore-schema", "ignore-table", "connect-options"} {
+	for _, persistOpt := range []string{"user", "ignore-schema", "ignore-table", "ignore-proc", "ignore-func", "connect-options"} {
 		if cfg.OnCLI(persistOpt) {
 			hostOptionFile.SetOptionValue(environment, persistOpt, cfg.Get(persistOpt))
 		}
@@ -193,7 +194,7 @@ func generatorString() string {
 // correct schema name.
 func PopulateSchemaDir(s *tengo.Schema, parentDir *fs.Dir, makeSubdir bool) error {
 	// Ignore any attempt to populate a dir for the temp schema
-	if s.Name == parentDir.Config.Get("temp-schema") {
+	if s.Name == parentDir.Config.GetAllowEnvVar("temp-schema") {
 		return nil
 	}
 
@@ -223,10 +224,6 @@ func PopulateSchemaDir(s *tengo.Schema, parentDir *fs.Dir, makeSubdir bool) erro
 	dumpOpts := dumper.Options{
 		IncludeAutoInc: dir.Config.GetBool("include-auto-inc"),
 	}
-	dumpOpts.IgnoreTable, err = dir.Config.GetRegexp("ignore-table")
-	if err != nil {
-		return NewExitValue(CodeBadConfig, err.Error())
-	}
 	if dir.Config.GetBool("strip-partitioning") {
 		dumpOpts.Partitioning = tengo.PartitioningRemove
 	}
