@@ -54,12 +54,7 @@ func LintHandler(cfg *mybase.Config) error {
 	result := lintWalker(dir, 5)
 	switch {
 	case len(result.Exceptions) > 0:
-		exitCode := CodeFatalError
-		for _, err := range result.Exceptions {
-			if _, ok := err.(linter.ConfigError); ok {
-				exitCode = CodeBadConfig
-			}
-		}
+		exitCode := ExitCode(HighestExitCode(result.Exceptions...))
 		return NewExitValue(exitCode, "Skipped %s due to fatal errors",
 			countAndNoun(len(result.Exceptions), "operation", "operations"),
 		)
@@ -141,7 +136,7 @@ func lintDir(dir *fs.Dir) *linter.Result {
 			if err != nil {
 				return linter.BadConfigResult(dir, err)
 			} else if inst == nil {
-				return linter.BadConfigResult(dir, fmt.Errorf("With workspace=temp-schema, this command needs a host to operate on, but no host is defined for environment %q", dir.Config.Get("environment")))
+				return linter.BadConfigResult(dir, fmt.Errorf("This command needs either a host (with workspace=temp-schema) or flavor (with workspace=docker), but one is not configured for environment %q", dir.Config.Get("environment")))
 			}
 		}
 		if wsOpts, err = workspace.OptionsForDir(dir, inst); err != nil {
@@ -166,7 +161,6 @@ func lintDir(dir *fs.Dir) *linter.Result {
 		if dir.Config.GetBool("format") && n == 0 {
 			dumpOpts := dumper.Options{
 				IncludeAutoInc: true,
-				IgnoreTable:    opts.IgnoreTable,
 			}
 			if dir.Config.GetBool("strip-partitioning") {
 				dumpOpts.Partitioning = tengo.PartitioningRemove
@@ -191,7 +185,7 @@ func lintDir(dir *fs.Dir) *linter.Result {
 	// Add warning annotations for unparseable statements (unless we hit an
 	// exception, in which case skip it to avoid extra noise!)
 	if len(result.Exceptions) == 0 {
-		for _, stmt := range dir.IgnoredStatements {
+		for _, stmt := range dir.UnparsedStatements {
 			note := linter.Note{
 				Summary: "Unable to parse statement",
 				Message: "Ignoring unsupported or unparseable SQL statement",
