@@ -4,7 +4,6 @@ import (
 	"database/sql"
 	"fmt"
 	"os"
-	"regexp"
 
 	log "github.com/sirupsen/logrus"
 	"github.com/skeema/mybase"
@@ -139,11 +138,8 @@ func pullSchemaDir(dir *fs.Dir, instance *tengo.Instance) (schemaNames []string,
 		// TODO: support multiple logical schemas per dir
 		logicalSchema := dir.LogicalSchemas[0]
 		schemaNames, err = pullLogicalSchema(dir, instance, logicalSchema)
-	}
-	if err != nil {
-		log.Errorf("Skipping %s: %s\n", dir, err)
-		if _, ok := err.(*ExitValue); !ok {
-			err = NewExitValue(CodeFatalError, "")
+		if err != nil {
+			log.Errorf("Skipping %s: %s\n", dir, err)
 		}
 	}
 	return
@@ -164,7 +160,7 @@ func pullLogicalSchema(dir *fs.Dir, instance *tengo.Instance, logicalSchema *fs.
 	if logicalSchema.Name != "" {
 		schemaNames = []string{logicalSchema.Name}
 	} else if schemaNames, err = dir.SchemaNames(instance); err != nil {
-		return nil, fmt.Errorf("unable to fetch schema names mapped by this dir: %s", err)
+		return nil, fmt.Errorf("unable to fetch schema names mapped by this dir: %w", err)
 	}
 	if len(schemaNames) == 0 {
 		log.Warnf("Ignoring directory %s -- did not map to any schema names for environment %q\n", dir, dir.Config.Get("environment"))
@@ -177,6 +173,7 @@ func pullLogicalSchema(dir *fs.Dir, instance *tengo.Instance, logicalSchema *fs.
 	} else if err != nil {
 		return nil, fmt.Errorf("Unable to fetch schema %s from %s: %s", schemaNames[0], instance, err)
 	}
+	instSchema.StripMatches(dir.IgnorePatterns)
 
 	log.Infof("Updating %s to reflect %s %s", dir, instance, instSchema.Name)
 
@@ -189,9 +186,6 @@ func pullLogicalSchema(dir *fs.Dir, instance *tengo.Instance, logicalSchema *fs.
 	dumpOpts := dumper.Options{
 		IncludeAutoInc: dir.Config.GetBool("include-auto-inc"),
 	}
-	if dumpOpts.IgnoreTable, err = dir.Config.GetRegexp("ignore-table"); err != nil {
-		return nil, NewExitValue(CodeBadConfig, err.Error())
-	}
 	if !dir.Config.GetBool("update-partitioning") {
 		if dir.Config.GetBool("strip-partitioning") {
 			// Undocumented due to potential confusion, but supported just like in init
@@ -208,7 +202,7 @@ func pullLogicalSchema(dir *fs.Dir, instance *tengo.Instance, logicalSchema *fs.
 	// To make this distinction, we need to actually execute the *.sql files in a
 	// Workspace and run a diff against it.
 	if !dir.Config.GetBool("format") || !dir.Config.GetBool("normalize") {
-		mods := statementModifiersForPull(dir.Config, instance, dumpOpts.IgnoreTable)
+		mods := statementModifiersForPull(dir.Config, instance)
 		opts, err := workspace.OptionsForDir(dir, instance)
 		if err != nil {
 			return nil, NewExitValue(CodeBadConfig, err.Error())
@@ -227,7 +221,7 @@ func pullLogicalSchema(dir *fs.Dir, instance *tengo.Instance, logicalSchema *fs.
 	return
 }
 
-func statementModifiersForPull(config *mybase.Config, instance *tengo.Instance, ignoreTable *regexp.Regexp) tengo.StatementModifiers {
+func statementModifiersForPull(config *mybase.Config, instance *tengo.Instance) tengo.StatementModifiers {
 	// We're permissive of unsafe operations here since we don't ever actually
 	// execute the generated statement! We just examine its type.
 	mods := tengo.StatementModifiers{
@@ -240,7 +234,6 @@ func statementModifiersForPull(config *mybase.Config, instance *tengo.Instance,
 	} else {
 		mods.NextAutoInc = tengo.NextAutoIncIfAlready
 	}
-	mods.IgnoreTable = ignoreTable
 	instFlavor, confFlavor := instance.Flavor(), tengo.ParseFlavor(config.Get("flavor"))
 	if !instFlavor.Known() && confFlavor.Known() {
 		mods.Flavor = confFlavor
@@ -274,6 +267,7 @@ func objectsInDiff(logicalSchema *fs.LogicalSchema, instSchema *tengo.Schema, op
 	inDiff := make([]tengo.ObjectKey, 0)
 	for _, od := range diff.ObjectDiffs() {
 		odStatement, odStatementErr := od.Statement(mods)
+		key := od.ObjectKey()
 		// Errors are fatal, except for UnsupportedDiffError which we can safely
 		// ignore (since pull doesn't actually run ALTERs; it just needs to know
 		// what was altered)
@@ -283,7 +277,7 @@ func objectsInDiff(logicalSchema *fs.LogicalSchema, instSchema *tengo.Schema, op
 		// mods may cause the diff to be a no-op; only include it in result if this
 		// isn't the case
 		if odStatement != "" {
-			inDiff = append(inDiff, od.ObjectKey())
+			inDiff = append(inDiff, key)
 		}
 	}
 
@@ -355,6 +349,7 @@ func findNewSchemas(dir *fs.Dir, instance *tengo.Instance, seenNames []string) e
 			if err != nil {
 				return err
 			}
+			s.StripMatches(dir.IgnorePatterns)
 			// use same logic from init command
 			if err := PopulateSchemaDir(s, dir, true); err != nil {
 				return err
