@@ -182,9 +182,6 @@ func StatementModifiersForDir(dir *fs.Dir) (mods tengo.StatementModifiers, err e
 	if mods.LockClause, err = dir.Config.GetEnum("alter-lock", "none", "shared", "exclusive", "default"); err != nil {
 		return
 	}
-	if mods.IgnoreTable, err = dir.Config.GetRegexp("ignore-table"); err != nil {
-		return
-	}
 	var partitioning string
 	if partitioning, err = dir.Config.GetEnum("partitioning", "keep", "remove", "modify"); err != nil {
 		return
@@ -205,3 +202,9 @@ type ConfigError string
 func (ce ConfigError) Error() string {
 	return string(ce)
 }
+
+// ExitCode returns 78 for ConfigError, corresponding to EX_CONFIG in BSD's
+// SYSEXITS(3) manpage.
+func (ce ConfigError) ExitCode() int {
+	return 78
+}
