@@ -21,14 +21,15 @@ import (
 // Dir is a parsed representation of a directory that may have contained
 // a .skeema config file and/or *.sql files.
 type Dir struct {
-	Path              string
-	Config            *mybase.Config
-	OptionFile        *mybase.File
-	SQLFiles          []SQLFile
-	LogicalSchemas    []*LogicalSchema // for now, always 0 or 1 elements; 2+ in same dir to be supported in future
-	ParseError        error            // any fatal error found parsing dir's config or contents
-	IgnoredStatements []*Statement     // statements with unknown type / not supported by this package
-	repoBase          string           // absolute path of containing repo, or topmost-found .skeema file
+	Path               string
+	Config             *mybase.Config
+	OptionFile         *mybase.File
+	SQLFiles           []SQLFile
+	LogicalSchemas     []*LogicalSchema      // for now, always 0 or 1 elements; 2+ in same dir to be supported in future
+	ParseError         error                 // any fatal error found parsing dir's config or contents
+	UnparsedStatements []*Statement          // statements with unknown type / not supported by this package
+	IgnorePatterns     []tengo.ObjectPattern // regexes for matching objects that should be ignored
+	repoBase           string                // absolute path of containing repo, or topmost-found .skeema file
 }
 
 // LogicalSchema represents a set of statements from *.sql files in a directory
@@ -224,9 +225,9 @@ func (dir *Dir) Subdirs() ([]*Dir, error) {
 func (dir *Dir) CreateSubdir(name string, optionFile *mybase.File) (*Dir, error) {
 	dirPath := filepath.Join(dir.Path, name)
 	if dir.OptionFile != nil && dir.OptionFile.SomeSectionHasOption("schema") {
-		return nil, fmt.Errorf("Cannot use dir %s: parent option file %s defines schema option", dirPath, dir.OptionFile)
+		return nil, ConfigErrorf("Cannot use dir %s: parent option file %s defines schema option", dirPath, dir.OptionFile)
 	} else if _, ok := dir.Config.Source("schema").(*mybase.File); ok {
-		return nil, fmt.Errorf("Cannot use dir %s: an ancestor option file defines schema option", dirPath)
+		return nil, ConfigErrorf("Cannot use dir %s: an ancestor option file defines schema option", dirPath)
 	}
 
 	if fi, err := os.Stat(dirPath); os.IsNotExist(err) {
@@ -293,11 +294,11 @@ func (dir *Dir) CreateOptionFile(optionFile *mybase.File) (err error) {
 func (dir *Dir) Hostnames() ([]string, error) {
 	if dir.Config.Changed("host-wrapper") {
 		variables := map[string]string{
-			"HOST":        dir.Config.Get("host"),
+			"HOST":        dir.Config.GetAllowEnvVar("host"),
 			"ENVIRONMENT": dir.Config.Get("environment"),
 			"DIRNAME":     dir.BaseName(),
 			"DIRPATH":     dir.Path,
-			"SCHEMA":      dir.Config.Get("schema"),
+			"SCHEMA":      dir.Config.GetAllowEnvVar("schema"),
 		}
 		shellOut, err := util.NewInterpolatedShellOut(dir.Config.Get("host-wrapper"), variables)
 		if err != nil {
@@ -305,7 +306,18 @@ func (dir *Dir) Hostnames() ([]string, error) {
 		}
 		return shellOut.RunCaptureSplit()
 	}
-	return dir.Config.GetSlice("host", ',', true), nil
+	return dir.Config.GetSliceAllowEnvVar("host", ',', true), nil
+}
+
+// Port returns the port number in the directory's configuration (often the
+// default of 3306) and a boolean indicating whether the port was configured
+// explicitly using the port option.
+func (dir *Dir) Port() (int, bool) {
+	intValue, _ := strconv.Atoi(dir.Config.GetAllowEnvVar("port"))
+	if intValue == 0 {
+		intValue = 3306
+	}
+	return intValue, dir.Config.Supplied("port")
 }
 
 // Instances returns 0 or more tengo.Instance pointers, based on the
@@ -326,20 +338,23 @@ func (dir *Dir) Instances() ([]*tengo.Instance, error) {
 
 	// Before looping over hostnames, do a single lookup of user, password,
 	// connect-options, port, socket.
+	user := dir.Config.GetAllowEnvVar("user")
+	password, err := dir.Password(hosts...)
+	if err != nil {
+		return nil, err // for example, need interactive password but STDIN isn't a TTY
+	}
 	var userAndPass string
-	if !dir.Config.Changed("password") {
-		userAndPass = dir.Config.Get("user")
+	if password == "" {
+		userAndPass = user
 	} else {
-		userAndPass = fmt.Sprintf("%s:%s", dir.Config.Get("user"), dir.Config.Get("password"))
+		userAndPass = user + ":" + password
 	}
 	params, err := dir.InstanceDefaultParams()
 	if err != nil {
-		return nil, fmt.Errorf("Invalid connection options: %s", err)
+		return nil, ConfigErrorf("Invalid connection options: %w", err)
 	}
-	portValue := dir.Config.GetIntOrDefault("port")
-	portWasSupplied := dir.Config.Supplied("port")
-	portIsntDefault := dir.Config.Changed("port")
-	socketValue := dir.Config.Get("socket")
+	portValue, portWasSupplied := dir.Port()
+	socketValue := dir.Config.GetAllowEnvVar("socket")
 	socketWasSupplied := dir.Config.Supplied("socket")
 
 	// For each hostname, construct a DSN and use it to create an Instance
@@ -355,8 +370,8 @@ func (dir *Dir) Instances() ([]*tengo.Instance, error) {
 				return nil, err
 			}
 			if splitPort > 0 {
-				if portIsntDefault && portValue != splitPort {
-					return nil, fmt.Errorf("Port was supplied as %d inside hostname %s but as %d in option file", splitPort, host, portValue)
+				if splitPort != portValue && portWasSupplied {
+					return nil, ConfigErrorf("Port was supplied as %d inside hostname %s but as %d in option file", splitPort, host, portValue)
 				}
 				host = splitHost
 				thisPortValue = splitPort
@@ -366,11 +381,11 @@ func (dir *Dir) Instances() ([]*tengo.Instance, error) {
 		dsn := fmt.Sprintf("%s@%s(%s)/?%s", userAndPass, net, addr, params)
 		instance, err := util.NewInstance("mysql", dsn)
 		if err != nil {
-			if dir.Config.Changed("password") {
-				safeUserPass := fmt.Sprintf("%s:*****", dir.Config.Get("user"))
+			if password != "" {
+				safeUserPass := user + ":*****"
 				dsn = strings.Replace(dsn, userAndPass, safeUserPass, 1)
 			}
-			return nil, fmt.Errorf("Invalid connection information for %s (DSN=%s): %s", dir, dsn, err)
+			return nil, ConfigErrorf("Invalid connection information for %s (DSN=%s): %w", dir, dsn, err)
 		}
 		instances = append(instances, instance)
 	}
@@ -437,18 +452,18 @@ func (dir *Dir) SchemaNames(instance *tengo.Instance) (names []string, err error
 	// If no schema defined in this dir (meaning this dir's .skeema, as well as
 	// parent dirs' .skeema, global option files, or command-line) for the current
 	// environment, then nothing to do
-	if !dir.Config.Changed("schema") {
+	schemaValue := dir.Config.GetAllowEnvVar("schema") // Strips quotes (including backticks) from fully quoted-wrapped values
+	if schemaValue == "" {
 		return nil, nil
 	}
 
-	schemaValue := dir.Config.Get("schema")                        // Get strips quotes (including backticks) from fully quoted-wrapped values
-	rawSchemaValue := dir.Config.GetRaw("schema")                  // GetRaw does not strip quotes
-	if rawSchemaValue != schemaValue && rawSchemaValue[0] == '`' { // no need to check len, the Changed check above already tells us schema != ""
+	rawSchemaValue := dir.Config.GetRaw("schema")                  // Does not strip quotes
+	if rawSchemaValue != schemaValue && rawSchemaValue[0] == '`' { // no need to check len: since non-raw value isn't empty, raw value can't be empty
 		variables := map[string]string{
 			"HOST":        instance.Host,
 			"PORT":        strconv.Itoa(instance.Port),
-			"USER":        dir.Config.Get("user"),
-			"PASSWORD":    dir.Config.Get("password"),
+			"USER":        dir.Config.GetAllowEnvVar("user"),
+			"PASSWORD":    dir.Config.GetAllowEnvVar("password"),
 			"ENVIRONMENT": dir.Config.Get("environment"),
 			"DIRNAME":     dir.BaseName(),
 			"DIRPATH":     dir.Path,
@@ -474,7 +489,7 @@ func (dir *Dir) SchemaNames(instance *tengo.Instance) (names []string, err error
 		if schemaValue != "*" {
 			re, err := regexp.Compile(schemaValue[1 : len(schemaValue)-1])
 			if err != nil {
-				return nil, err
+				return nil, ConfigError{err}
 			}
 			keepNames := []string{}
 			for _, name := range names {
@@ -485,7 +500,7 @@ func (dir *Dir) SchemaNames(instance *tengo.Instance) (names []string, err error
 			names = keepNames
 		}
 	} else {
-		names = dir.Config.GetSlice("schema", ',', true)
+		names = dir.Config.GetSliceAllowEnvVar("schema", ',', true)
 	}
 
 	// Remove ignored schemas and system schemas. (tengo removes the latter from
@@ -493,7 +508,7 @@ func (dir *Dir) SchemaNames(instance *tengo.Instance) (names []string, err error
 	// can't manually configure the schema option to a system schema.)
 	ignoreSchema, err := dir.Config.GetRegexp("ignore-schema")
 	if err != nil {
-		return nil, err
+		return nil, ConfigError{err}
 	}
 	names = filterSchemaNames(names, ignoreSchema)
 
@@ -618,7 +633,7 @@ func (dir *Dir) InstanceDefaultParams() (string, error) {
 
 	options, err := util.SplitConnectOptions(dir.Config.Get("connect-options"))
 	if err != nil {
-		return "", err
+		return "", ConfigError{err}
 	}
 
 	v := url.Values{}
@@ -633,7 +648,7 @@ func (dir *Dir) InstanceDefaultParams() (string, error) {
 	if dir.Config.Supplied("ssl-mode") {
 		sslMode, err = dir.Config.GetEnum("ssl-mode", "disabled", "preferred", "required")
 		if err != nil {
-			return "", err
+			return "", ConfigError{err}
 		} else if sslMode == "disabled" {
 			sslMode = "false" // driver uses "false" to mean mysql ssl-mode=disabled
 		} else if sslMode == "required" {
@@ -647,10 +662,10 @@ func (dir *Dir) InstanceDefaultParams() (string, error) {
 	// Set values from connect-options
 	for name, value := range options {
 		if banned[strings.ToLower(name)] {
-			return "", fmt.Errorf("connect-options is not allowed to contain %s", name)
+			return "", ConfigErrorf("connect-options is not allowed to contain %s", name)
 		}
 		if name == "tls" && dir.Config.Supplied("ssl-mode") {
-			return "", fmt.Errorf("connect-options is not allowed to contain %s; use only the newer ssl-mode option instead", name)
+			return "", ConfigErrorf("connect-options is not allowed to contain %s; use only the newer ssl-mode option instead", name)
 		}
 		v.Set(name, value)
 	}
@@ -675,6 +690,87 @@ func (dir *Dir) Generator() (major, minor, patch int, edition string) {
 	return int(version.Major()), int(version.Minor()), int(version.Patch()), edition
 }
 
+// Package-level user@host interactive password cache, used by Dir.Password()
+var cachedInteractivePasswords = make(map[string]string)
+
+// Password returns the configured password in this dir, a cached password
+// from a previous interactive password check, or an interactively-prompted
+// password from STDIN if one should be obtained based on the directory's
+// configuration. If interactive input is requested and successful, the password
+// will be returned and also cached, so that subsequent identical requests
+// return the password without prompting.
+//
+// Optionally supply one or more hostnames to affect the behavior of interactive
+// password prompts and caching: with no hosts, the prompt text will mention the
+// directory and be cached in the directory's configuration; with one or more
+// hosts, the prompt text will mention the first host and will cache values in
+// a package-level map independent of this dir.
+//
+// An error is returned if a password should be prompted but cannot, for example
+// due to STDIN not being a TTY.
+func (dir *Dir) Password(hosts ...string) (string, error) {
+	// Only prompt if password option was supplied with no equals sign or value.
+	// If it was supplied with an equals sign but set to a blank value, mybase
+	// will expose this as "''" from GetRaw, since GetRaw doesn't remove the quotes
+	// like other Config getters. This allows us to differentiate between "prompt
+	// on STDIN" and "intentionally no/blank password" situations.
+	if dir.Config.GetRaw("password") != "" {
+		return dir.Config.GetAllowEnvVar("password"), nil
+	}
+
+	cacheKeys := make([]string, len(hosts))
+	var promptArg string
+	if len(hosts) == 0 {
+		// No need to check a cache for dir-level prompting, since the previous Config
+		// check will already have managed a previously-prompted password
+		promptArg = "directory " + dir.RelPath()
+	} else {
+		user := dir.Config.GetAllowEnvVar("user")
+		for n, host := range hosts {
+			cacheKeys[n] = user + "@" + host
+			if cachedPassword, ok := cachedInteractivePasswords[cacheKeys[n]]; ok {
+				return cachedPassword, nil
+			}
+		}
+		promptArg = cacheKeys[0]
+		if len(hosts) == 2 {
+			promptArg += " and " + cacheKeys[1]
+		} else if len(hosts) > 2 {
+			promptArg = fmt.Sprintf("%s and %d other servers", promptArg, len(hosts)-1)
+		}
+	}
+
+	val, err := util.PromptPassword("Enter password for %s: ", promptArg)
+	if err != nil {
+		return "", fmt.Errorf("Unable to prompt password for %s: %w", promptArg, err)
+	}
+
+	if len(hosts) == 0 {
+		// We single-quote-wrap the value (escaping any internal single-quotes) to
+		// prevent a redundant pw prompt on an empty string, and also to prevent
+		// input of the form $SOME_ENV_VAR from performing env var substitution.
+		cacheVal := fmt.Sprintf("'%s'", strings.ReplaceAll(val, "'", "\\'"))
+		dir.Config.SetRuntimeOverride("password", cacheVal)
+	}
+	for _, cacheKey := range cacheKeys {
+		// For caching per host, we use the value as-is since this does not go
+		// through mybase Config getters.
+		cachedInteractivePasswords[cacheKey] = val
+	}
+	return val, nil
+}
+
+// ShouldIgnore returns true if the directory's configuration states that the
+// supplied object/key/statement should be ignored.
+func (dir *Dir) ShouldIgnore(object tengo.ObjectKeyer) bool {
+	for _, pattern := range dir.IgnorePatterns {
+		if pattern.Match(object) {
+			return true
+		}
+	}
+	return false
+}
+
 // parseContents reads the .skeema and *.sql files in the dir, populating
 // fields of dir accordingly. This method modifies dir in-place. Any fatal
 // error will populate dir.ParseError.
@@ -690,6 +786,12 @@ func (dir *Dir) parseContents() {
 		dir.Config.AddSource(dir.OptionFile)
 	}
 
+	var err error
+	if dir.IgnorePatterns, err = util.IgnorePatterns(dir.Config); err != nil {
+		dir.ParseError = ConfigError{err}
+		return
+	}
+
 	// Tokenize and parse any *.sql files
 	if dir.SQLFiles, dir.ParseError = sqlFiles(dir.Path, dir.repoBase); dir.ParseError != nil {
 		return
@@ -708,6 +810,13 @@ func (dir *Dir) parseContents() {
 			return
 		}
 		for _, stmt := range tokenizedFile.Statements {
+			// Statements that are ignored due to ignore-table, ignore-proc, etc are
+			// simply not placed into a LogicalSchema, so that all other logic won't
+			// interact with them
+			if dir.ShouldIgnore(stmt) {
+				continue
+			}
+
 			if _, ok := logicalSchemasByName[stmt.Schema()]; !ok {
 				logicalSchemasByName[stmt.Schema()] = &LogicalSchema{
 					Creates: make(map[tengo.ObjectKey]*Statement),
@@ -722,7 +831,7 @@ func (dir *Dir) parseContents() {
 				// type (e.g. INSERTs), are simply ignored. This is not fatal, since it is
 				// quite rare for a typo to trigger this -- only happens when misspelling
 				// CREATE or the object type for example.
-				dir.IgnoredStatements = append(dir.IgnoredStatements, stmt)
+				dir.UnparsedStatements = append(dir.UnparsedStatements, stmt)
 			} else if stmt.Type == StatementTypeLexError || stmt.Type == StatementTypeForbidden {
 				// Statements with lexer errors, meaning invalid characters, are treated as
 				// fatal. This can be indicative of a bug in the grammar, or of a normally-
@@ -764,6 +873,18 @@ func (dir *Dir) parseContents() {
 			dir.LogicalSchemas = append(dir.LogicalSchemas, ls)
 		}
 	}
+
+	// If the dir's configuration includes "password" with no =value, and the dir
+	// does not configure any hosts, prompt for password now. This way, any subdirs
+	// will inherit the password without having to each prompt individually.
+	if !dir.Config.Changed("host") {
+		// This has no side-effects if the dir isn't configured to prompt for pw
+		// interactively. It will only return an error if an interactive prompt
+		// is attempted but fails due to STDIN not being a TTY.
+		if _, err := dir.Password(); err != nil {
+			log.Warn(err)
+		}
+	}
 }
 
 // ParentOptionFiles returns a slice of *mybase.File, corresponding to the
@@ -905,7 +1026,7 @@ func parseOptionFile(dirPath, repoBase string, baseConfig *mybase.Config) (*myba
 		return nil, err
 	}
 	if err := f.Parse(baseConfig); err != nil {
-		return nil, err
+		return nil, ConfigError{err}
 	}
 	_ = f.UseSection(baseConfig.Get("environment")) // we don't care if the section doesn't exist
 	return f, nil
@@ -957,6 +1078,33 @@ func sqlFiles(dirPath, repoBase string) ([]SQLFile, error) {
 	return result, nil
 }
 
+// ConfigError indicates a misconfiguration in the directory's .skeema file or
+// the command-line overrides.
+type ConfigError struct {
+	err error
+}
+
+// Error satisfies the builtin error interface.
+func (ce ConfigError) Error() string {
+	return ce.err.Error()
+}
+
+// Unwrap satisfies Golang errors package unwrapping behavior.
+func (ce ConfigError) Unwrap() error {
+	return ce.err
+}
+
+// ExitCode returns 78 for ConfigError, corresponding to EX_CONFIG in BSD's
+// SYSEXITS(3) manpage.
+func (ce ConfigError) ExitCode() int {
+	return 78
+}
+
+// ConfigErrorf formats and returns a new ConfigError value.
+func ConfigErrorf(format string, a ...any) ConfigError {
+	return ConfigError{err: fmt.Errorf(format, a...)}
+}
+
 // DuplicateDefinitionError is an error returned when Dir.parseContents()
 // encounters multiple CREATE statements for the same exact object.
 type DuplicateDefinitionError struct {
