@@ -1,6 +1,8 @@
 package fs
 
 import (
+	"errors"
+	"fmt"
 	"net/url"
 	"os"
 	"path/filepath"
@@ -22,8 +24,8 @@ func TestParseDir(t *testing.T) {
 	if dir.Config.Get("host") != "127.0.0.1" {
 		t.Errorf("dir.Config not working as expected; host is %s", dir.Config.Get("host"))
 	}
-	if len(dir.IgnoredStatements) > 0 {
-		t.Errorf("Expected 0 IgnoredStatements, instead found %d", len(dir.IgnoredStatements))
+	if len(dir.UnparsedStatements) > 0 {
+		t.Errorf("Expected 0 UnparsedStatements, instead found %d", len(dir.UnparsedStatements))
 	}
 	if len(dir.LogicalSchemas) != 1 {
 		t.Fatalf("Expected 1 LogicalSchema; instead found %d", len(dir.LogicalSchemas))
@@ -289,17 +291,17 @@ func TestParseDirInvalidChar(t *testing.T) {
 	}
 }
 
-func TestParseDirUnknownIgnored(t *testing.T) {
+func TestParseDirUnparsedStatements(t *testing.T) {
 	// This dir contains an INSERT statement among the valid CREATEs. This should
-	// be tracked in dir.IgnoredStatements but isn't a fatal error.
+	// be tracked in dir.UnparsedStatements but isn't a fatal error.
 	if dir, err := ParseDir("testdata/unknownstatement", getValidConfig(t)); err != nil {
 		t.Fatalf("In dir testdata/unknownstatement, unexpected error from ParseDir(): %v", err)
 	} else if len(dir.LogicalSchemas) != 1 {
 		t.Fatalf("In dir testdata/unknownstatement, expected 1 logical schema, instead found %d", len(dir.LogicalSchemas))
 	} else if dir.ParseError != nil {
 		t.Fatalf("In dir testdata/unknownstatement, expected nil ParseError, instead found %v", dir.ParseError)
-	} else if len(dir.IgnoredStatements) != 1 {
-		t.Errorf("In dir testdata/unknownstatement, expected 1 IgnoredStatements, instead found %d", len(dir.IgnoredStatements))
+	} else if len(dir.UnparsedStatements) != 1 {
+		t.Errorf("In dir testdata/unknownstatement, expected 1 UnparsedStatements, instead found %d", len(dir.UnparsedStatements))
 	}
 }
 
@@ -307,7 +309,7 @@ func TestParseDirRedundantDelimiter(t *testing.T) {
 	// This dir contains two special cases of DELIMITER commands:
 	// * Setting a delimiter that is already the current delimiter, e.g. from ; to ;
 	// * Setting a delimiter that is double the previous delimiter, e.g. from ; to ;;
-	// These cases should not cause errors or IgnoredStatements.
+	// These cases should not cause errors or UnparsedStatements.
 	if dir, err := ParseDir("testdata/redundantdelimiter", getValidConfig(t)); err != nil {
 		t.Fatalf("In dir testdata/redundantdelimiter, unexpected error from ParseDir(): %v", err)
 	} else if len(dir.LogicalSchemas) != 1 {
@@ -316,8 +318,8 @@ func TestParseDirRedundantDelimiter(t *testing.T) {
 		t.Fatalf("In dir testdata/redundantdelimiter, expected 3 CREATEs, instead found %d", len(dir.LogicalSchemas[0].Creates))
 	} else if dir.ParseError != nil {
 		t.Fatalf("In dir testdata/redundantdelimiter, expected nil ParseError, instead found %v", dir.ParseError)
-	} else if len(dir.IgnoredStatements) > 0 {
-		t.Errorf("In dir testdata/redundantdelimiter, expected 0 IgnoredStatements, instead found %d, first is %+v", len(dir.IgnoredStatements), *dir.IgnoredStatements[0])
+	} else if len(dir.UnparsedStatements) > 0 {
+		t.Errorf("In dir testdata/redundantdelimiter, expected 0 UnparsedStatements, instead found %d, first is %+v", len(dir.UnparsedStatements), *dir.UnparsedStatements[0])
 	}
 }
 
@@ -342,8 +344,34 @@ func TestParseDirBOM(t *testing.T) {
 		t.Fatalf("In dir testdata/utf8bom, expected 2 CREATEs, instead found %d", len(dir.LogicalSchemas[0].Creates))
 	} else if dir.ParseError != nil {
 		t.Fatalf("In dir testdata/utf8bom, expected nil ParseError, instead found %v", dir.ParseError)
-	} else if len(dir.IgnoredStatements) != 0 {
-		t.Errorf("In dir testdata/utf8bom, expected 0 IgnoredStatements, instead found %d", len(dir.IgnoredStatements))
+	} else if len(dir.UnparsedStatements) != 0 {
+		t.Errorf("In dir testdata/utf8bom, expected 0 UnparsedStatements, instead found %d", len(dir.UnparsedStatements))
+	}
+}
+
+func TestParseDirIgnorePatterns(t *testing.T) {
+	// Confirm behavior of ignore pattern blocking all procs
+	dir := getDir(t, "testdata/ignore/invalidsql")
+	if len(dir.LogicalSchemas) != 1 {
+		t.Errorf("Expected 1 logical schema, instead found %d", len(dir.LogicalSchemas))
+	}
+	if len(dir.LogicalSchemas[0].Creates) != 2 {
+		t.Errorf("Expected 2 non-ignored CREATES in logical schema, instead found %d", len(dir.LogicalSchemas[0].Creates))
+	}
+	for key := range dir.LogicalSchemas[0].Creates {
+		if key.Type == tengo.ObjectTypeProc {
+			t.Errorf("Expected all procs to be ignored by ignore-proc=., but found proc with name %s", key.Name)
+		}
+	}
+
+	// Confirm behavior of invalid regex for ignore pattern
+	_, err := ParseDir("testdata/ignore/invalidregex", getValidConfig(t))
+	if err == nil {
+		t.Fatal("In dir testdata/ignore/invalidregex, expected error from ParseDir(), but instead err is nil")
+	}
+	var ce ConfigError
+	if !errors.As(err, &ce) {
+		t.Errorf("Expected err to be ConfigError, instead type is %T and it does not unwrap to ConfigError", err)
 	}
 }
 
@@ -432,7 +460,7 @@ func TestDirInstances(t *testing.T) {
 	assertInstances(map[string]string{"host": "some.db.host:3307"}, false, "some.db.host:3307")
 	assertInstances(map[string]string{"host": "some.db.host", "port": "3307"}, false, "some.db.host:3307")
 	assertInstances(map[string]string{"host": "some.db.host:3307", "port": "3307"}, false, "some.db.host:3307")
-	assertInstances(map[string]string{"host": "some.db.host:3307", "port": "3306"}, false, "some.db.host:3307") // port option ignored if default, even if explicitly specified
+	assertInstances(map[string]string{"host": "some.db.host:3307", "port": "3306"}, true) // mismatched port option not ignored if supplied explicitly, even if default
 	assertInstances(map[string]string{"host": "localhost"}, false, "localhost:/tmp/mysql.sock")
 	assertInstances(map[string]string{"host": "localhost", "port": "1234"}, false, "localhost:1234")
 	assertInstances(map[string]string{"host": "localhost", "socket": "/var/run/mysql.sock"}, false, "localhost:/var/run/mysql.sock")
@@ -600,6 +628,121 @@ func TestAncestorPaths(t *testing.T) {
 	}
 }
 
+func TestDirPassword(t *testing.T) {
+	defer func() {
+		util.PasswordPromptInput = util.PasswordInputSource(util.NoInteractiveInput)
+		cachedInteractivePasswords = make(map[string]string)
+	}()
+
+	// If a parent dir .skeema file has a bare "password" line, pw should be
+	// prompted there, but not redundantly for its subdirs, since it gets cached
+	// in the dir's config as a runtime override, at dir parsing time (e.g. getDir)
+	util.PasswordPromptInput = util.NewMockPasswordInput("basedir")
+	dir := getDir(t, "testdata/pwprompt/basedir")
+	util.PasswordPromptInput = util.PasswordInputSource(util.NoInteractiveInput)
+	if pw, err := dir.Password(); pw != "basedir" || err != nil {
+		t.Errorf("Unexpected return values from dir.Password(): %q, %v", pw, err)
+	}
+	util.PasswordPromptInput = util.NewMockPasswordInput("different value to ensure not re-prompted")
+	subdirs, err := dir.Subdirs()
+	if err != nil {
+		t.Fatalf("Unexpected error from Subdirs: %v", err)
+	}
+	for _, subdir := range subdirs {
+		if pw, err := subdir.Password(); pw != "basedir" || err != nil {
+			t.Errorf("Unexpected return values from subdir.Password(): %q, %v", pw, err)
+		}
+	}
+
+	// Same situation as above, but verify that a blank interactive password won't
+	// re-prompt redundantly for subdirs
+	util.PasswordPromptInput = util.NewMockPasswordInput("")
+	dir = getDir(t, "testdata/pwprompt/basedir")
+	util.PasswordPromptInput = util.NewMockPasswordInput("different value to ensure not re-prompted")
+	if pw, err := dir.Password(); pw != "" || err != nil {
+		t.Errorf("Unexpected return values from dir.Password(): %q, %v", pw, err)
+	}
+	subdirs, err = dir.Subdirs()
+	if err != nil {
+		t.Fatalf("Unexpected error from Subdirs: %v", err)
+	}
+	for _, subdir := range subdirs {
+		if pw, err := subdir.Password(); pw != "" || err != nil {
+			t.Errorf("Unexpected return values from subdir.Password(): %q, %v", pw, err)
+		}
+	}
+
+	// If parent dir doesn't have bare "password" but both subdirs do, they should
+	// each prompt password separately
+	util.PasswordPromptInput = util.NewMockPasswordInput("basedir")
+	dir = getDir(t, "testdata/pwprompt/leafdir")
+	if pw, err := dir.Password(); pw != "" || err != nil {
+		t.Errorf("Unexpected return values from dir.Password(): %q, %v", pw, err)
+	}
+	var counter int
+	util.PasswordPromptInput = func() (string, error) {
+		val := fmt.Sprintf("leaf-%d", counter)
+		counter++
+		return val, nil
+	}
+	subdirs, err = dir.Subdirs()
+	if err != nil {
+		t.Fatalf("Unexpected error from Subdirs: %v", err)
+	}
+	for n, subdir := range subdirs {
+		if pw, err := subdir.Password(); pw != fmt.Sprintf("leaf-%d", n) || err != nil {
+			t.Errorf("Unexpected return values from subdir.Password(): %q, %v", pw, err)
+		}
+	}
+
+	// If an equals sign is present, but no value or an empty string, this means
+	// empty password (not prompt) for compat with MySQL client behavior
+	util.PasswordPromptInput = util.NewMockPasswordInput("this should not show up")
+	dir = getDir(t, "testdata/pwprompt/noprompt/a")
+	if pw, err := dir.Password(); pw != "" || err != nil {
+		t.Errorf("Unexpected return values from dir.Password(): %q, %v", pw, err)
+	}
+	dir = getDir(t, "testdata/pwprompt/noprompt/b")
+	if pw, err := dir.Password(); pw != "" || err != nil {
+		t.Errorf("Unexpected return values from dir.Password(): %q, %v", pw, err)
+	}
+
+	// Now test prompting with hostnames:
+	// The first dir here contains 3 hosts, but a prompt should only occur once.
+	// Set the mock input to return a value once, followed by errors on subsequent
+	// calls.
+	util.PasswordPromptInput = func() (string, error) {
+		util.PasswordPromptInput = util.NoInteractiveInput
+		return "success", nil
+	}
+	dir = getDir(t, "testdata/pwprompt/hosts/a")
+	if pw, err := dir.Password(dir.Config.GetSlice("host", ',', true)...); pw != "success" || err != nil {
+		t.Errorf("Unexpected return values from dir.Password(): %q, %v", pw, err)
+	}
+
+	// The second dir has only one host, but it's one that also existed in first
+	// dir, so its pw should be cached since it also has same username.
+	dir = getDir(t, "testdata/pwprompt/hosts/b")
+	if pw, err := dir.Password(dir.Config.GetSlice("host", ',', true)...); pw != "success" || err != nil {
+		t.Errorf("Unexpected return values from dir.Password(): %q, %v", pw, err)
+	}
+
+	// The third dir has a single host that also appeared in first dir, but a
+	// different user name, so pw prompt should error instead of using cache!
+	dir = getDir(t, "testdata/pwprompt/hosts/c")
+	if pw, err := dir.Password(dir.Config.GetSlice("host", ',', true)...); pw != "" || err == nil {
+		t.Errorf("Unexpected return values from dir.Password(): %q, %v", pw, err)
+	}
+
+	// The fourth dir has a single host that did not appear previously, altho
+	// same user name as first two dirs. PW prompt should error instead of using
+	// cache.
+	dir = getDir(t, "testdata/pwprompt/hosts/d")
+	if pw, err := dir.Password(dir.Config.GetSlice("host", ',', true)...); pw != "" || err == nil {
+		t.Errorf("Unexpected return values from dir.Password(): %q, %v", pw, err)
+	}
+}
+
 func getValidConfigWithCLI(t *testing.T, cliOptions string) *mybase.Config {
 	t.Helper()
 	cmd := mybase.NewCommand("fstest", "", "", nil)
