@@ -3,7 +3,6 @@ package linter
 import (
 	"fmt"
 	"reflect"
-	"regexp"
 	"strings"
 
 	"github.com/skeema/mybase"
@@ -42,7 +41,6 @@ func AddCommandOptions(cmd *mybase.Command) {
 type Options struct {
 	RuleSeverity map[string]Severity
 	RuleConfig   map[string]interface{}
-	IgnoreTable  *regexp.Regexp
 	Flavor       tengo.Flavor
 	onlyKeys     map[tengo.ObjectKey]bool // if map is non-nil, only format objects with true values
 }
@@ -93,23 +91,13 @@ func (opts *Options) Equals(other *Options) bool {
 	if opts.Flavor != other.Flavor {
 		return false
 	}
-	if opts.IgnoreTable == nil || other.IgnoreTable == nil {
-		return opts.IgnoreTable == other.IgnoreTable
-	}
-	return opts.IgnoreTable.String() == other.IgnoreTable.String()
+	return true
 }
 
 // shouldIgnore returns true if the option configuration indicates the supplied
 // object should be ignored.
 func (opts *Options) shouldIgnore(keyer tengo.ObjectKeyer) bool {
-	key := keyer.ObjectKey()
-	if key.Type == tengo.ObjectTypeTable && opts.IgnoreTable != nil && opts.IgnoreTable.MatchString(key.Name) {
-		return true
-	}
-	if opts.onlyKeys != nil && !opts.onlyKeys[key] {
-		return true
-	}
-	return false
+	return opts.onlyKeys != nil && !opts.onlyKeys[keyer.ObjectKey()]
 }
 
 // OptionsForDir returns Options based on the configuration in an fs.Dir,
@@ -121,12 +109,6 @@ func OptionsForDir(dir *fs.Dir) (Options, error) {
 		Flavor:       tengo.ParseFlavor(dir.Config.Get("flavor")),
 	}
 
-	var err error
-	opts.IgnoreTable, err = dir.Config.GetRegexp("ignore-table")
-	if err != nil {
-		return Options{}, ConfigError{Dir: dir, err: err}
-	}
-
 	// Populate opts.RuleSeverity from individual rule options
 	for name, r := range rulesByName {
 		// Treat falsey values (incl --skip- prefix) as SeverityIgnore
@@ -194,6 +176,17 @@ func (ce ConfigError) Error() string {
 	return ce.err.Error()
 }
 
+// Unwrap satisfies Golang errors package unwrapping behavior.
+func (ce ConfigError) Unwrap() error {
+	return ce.err
+}
+
+// ExitCode returns 78 for ConfigError, corresponding to EX_CONFIG in BSD's
+// SYSEXITS(3) manpage.
+func (ce ConfigError) ExitCode() int {
+	return 78
+}
+
 // NewConfigError creates a config error referring to the specified directory
 // and message.
 func NewConfigError(dir *fs.Dir, format string, a ...interface{}) ConfigError {
