@@ -71,15 +71,15 @@ func (s IntegrationSuite) TestCheckSchema(t *testing.T) {
 	wsSchema, err := workspace.ExecLogicalSchema(logicalSchema, wsOpts)
 	if err != nil {
 		t.Fatalf("Unexpected error from workspace.ExecLogicalSchema: %v", err)
-	} else if len(wsSchema.Failures) != 3 {
+	} else if len(wsSchema.Failures) != 2 {
 		// Here we just verify that no statements are unexpectedly failing, besides
-		// the 3 in validcfg/borked.sql. We don't otherwise annotate failures here;
-		// testing of that logic is handled in TestResultAnnotateStatementErrors()
-		// in result_test.go.
+		// the 3 in validcfg/borked.sql, one of which is ignored by ignore-table. We
+		// don't otherwise annotate failures here; testing of that logic is handled
+		// in TestResultAnnotateStatementErrors() in result_test.go.
 		for _, err := range wsSchema.Failures {
 			t.Errorf(err.Error())
 		}
-		t.Fatalf("Expected 3 creation failures from %s/*.sql, instead found %d", dir, len(wsSchema.Failures))
+		t.Fatalf("Expected 2 creation failures from %s/*.sql, instead found %d (see above errors)", dir, len(wsSchema.Failures))
 	}
 
 	result := CheckSchema(wsSchema, opts)
@@ -270,12 +270,45 @@ func (s IntegrationSuite) TestCheckSchemaUTF8MB3(t *testing.T) {
 	compareAnnotations(t, expected, result)
 }
 
+// TestCheckSchemaAllowAllDefiner provides additional coverage for the default
+// allow-definer / lint-definer logic's perf shortcut.
+func (s IntegrationSuite) TestCheckSchemaAllowAllDefiner(t *testing.T) {
+	dir := getDir(t, "testdata/routines")
+	opts, err := OptionsForDir(dir)
+	if err != nil {
+		t.Fatalf("Unexpected error from OptionsForDir: %v", err)
+	}
+
+	logicalSchema := dir.LogicalSchemas[0]
+	wsOpts, err := workspace.OptionsForDir(dir, s.d.Instance)
+	if err != nil {
+		t.Fatalf("Unexpected error from workspace.OptionsForDir: %v", err)
+	}
+	wsSchema, err := workspace.ExecLogicalSchema(logicalSchema, wsOpts)
+	if err != nil {
+		t.Fatalf("Unexpected error from workspace.ExecLogicalSchema: %v", err)
+	} else if len(wsSchema.Failures) != 0 {
+		t.Fatalf("Unexpectedly found %d workspace failures", len(wsSchema.Failures))
+	}
+
+	// There's intentionally no .skeema file here; force the flavor value
+	// corresponding to the current Dockerized test db here
+	opts.Flavor = s.d.Flavor()
+
+	// Should have no annotations at all!
+	result := CheckSchema(wsSchema, opts)
+	if len(result.Annotations) > 0 {
+		t.Errorf("Expected 0 annotations, instead found %d", len(result.Annotations))
+	}
+}
+
 func (s *IntegrationSuite) Setup(backend string) (err error) {
 	s.d, err = s.manager.GetOrCreateInstance(tengo.DockerizedInstanceOptions{
 		Name:              fmt.Sprintf("skeema-test-%s", tengo.ContainerNameForImage(backend)),
 		Image:             backend,
 		RootPassword:      "fakepw",
 		DefaultConnParams: "foreign_key_checks=0&sql_mode=%27NO_ENGINE_SUBSTITUTION%27", // disabling strict mode to allow zero dates in testdata
+		CommandArgs:       []string{"--skip-log-bin"},                                   // override MySQL 8 default of enabling binlog
 	})
 	if err != nil {
 		return err
