@@ -2,7 +2,6 @@ package tengo
 
 import (
 	"fmt"
-	"regexp"
 	"strings"
 
 	"github.com/pmezard/go-difflib/difflib"
@@ -76,7 +75,6 @@ type StatementModifiers struct {
 	AllowUnsafe            bool             // Whether to allow potentially-destructive DDL (drop table, drop column, modify col type, etc)
 	LockClause             string           // Include a LOCK=[value] clause in generated ALTER TABLE
 	AlgorithmClause        string           // Include an ALGORITHM=[value] clause in generated ALTER TABLE
-	IgnoreTable            *regexp.Regexp   // Generate blank DDL if table name matches this regexp
 	StrictIndexOrder       bool             // If true, maintain index order even in cases where there is no functional difference
 	StrictCheckOrder       bool             // If true, maintain check constraint order even though it never has a functional difference (only affects MariaDB)
 	StrictForeignKeyNaming bool             // If true, maintain foreign key definition even if differences are cosmetic (name change, RESTRICT vs NO ACTION, etc)
@@ -168,8 +166,8 @@ func compareRoutines(from, to *Schema) (routineDiffs []*RoutineDiff) {
 				// then-ADD, but characteristic-only changes could use ALTER FUNCTION /
 				// ALTER PROCEDURE instead.
 				routineDiffs = append(routineDiffs,
-					&RoutineDiff{From: fromRoutine, ForMetadata: metadataOnly},
-					&RoutineDiff{To: toRoutine, ForMetadata: metadataOnly},
+					&RoutineDiff{From: fromRoutine, ForReplace: true, ForMetadata: metadataOnly},
+					&RoutineDiff{To: toRoutine, ForReplace: true, ForMetadata: metadataOnly},
 				)
 			}
 		}
@@ -516,11 +514,6 @@ func (td *TableDiff) Statement(mods StatementModifiers) (string, error) {
 	if td == nil {
 		return "", nil
 	}
-	if mods.IgnoreTable != nil {
-		if (td.From != nil && mods.IgnoreTable.MatchString(td.From.Name)) || (td.To != nil && mods.IgnoreTable.MatchString(td.To.Name)) {
-			return "", nil
-		}
-	}
 
 	var err error
 	switch td.Type {
@@ -682,6 +675,7 @@ func (td *TableDiff) alterStatement(mods StatementModifiers) (string, error) {
 type RoutineDiff struct {
 	From        *Routine
 	To          *Routine
+	ForReplace  bool // if true, routine is being dropped/re-created to replace
 	ForMetadata bool // if true, routine is being replaced only to update creation-time metadata
 }
 
@@ -716,20 +710,36 @@ func (rd *RoutineDiff) DiffType() DiffType {
 // still be returned as-is, but the error will be non-nil. Be sure not to
 // ignore the error value of this method.
 func (rd *RoutineDiff) Statement(mods StatementModifiers) (string, error) {
+	if rd == nil {
+		return "", nil
+	}
+
 	// If we're replacing a routine only because its creation-time sql_mode or
 	// db collation has changed, only proceed if mods indicate we should. (This
 	// type of replacement is effectively opt-in because it is counter-intuitive
 	// and obscure.)
-	if rd != nil && rd.ForMetadata && !mods.CompareMetadata {
+	if rd.ForMetadata && !mods.CompareMetadata {
 		return "", nil
 	}
+
+	var comment string
+	mariaReplace := rd.ForReplace && mods.Flavor.IsMariaDB()
 	switch rd.DiffType() {
-	case DiffTypeNone:
-		return "", nil
 	case DiffTypeCreate:
-		return rd.To.CreateStatement, nil
+		if mariaReplace && rd.ForMetadata {
+			comment = fmt.Sprintf("# Replacing %s to update metadata\n", rd.ObjectKey())
+		}
+		stmt := rd.To.CreateStatement
+		if mariaReplace {
+			stmt = strings.Replace(stmt, "CREATE ", "CREATE OR REPLACE ", 1)
+		}
+		return comment + stmt, nil
 	case DiffTypeDrop:
-		var comment string
+		// MariaDB 10.1+ can use CREATE OR REPLACE, so omit any replacement-motivated
+		// DROP statements
+		if mariaReplace {
+			return "", nil
+		}
 		if rd.ForMetadata {
 			comment = fmt.Sprintf("# Dropping and re-creating %s to update metadata\n", rd.ObjectKey())
 		}
