@@ -2,7 +2,6 @@ package tengo
 
 import (
 	"fmt"
-	"regexp"
 	"strings"
 	"testing"
 )
@@ -546,7 +545,8 @@ func TestSchemaDiffRoutines(t *testing.T) {
 		t.Errorf("Modifier AllowUnsafe=true not working; error (%s) returned for %s", err, stmt)
 	}
 
-	// Test alter, which currently always is handled by a drop and re-add.
+	// Test alter, which is handled by a drop and re-add in MySQL/Percona, and
+	// OR REPLACE in MariaDB.
 	// Since this is a creation-time metadata change, also test statement modifier
 	// affecting whether or not those changes are suppressed.
 	s1r2 := aProc("utf8mb4_general_ci", "")
@@ -569,13 +569,14 @@ func TestSchemaDiffRoutines(t *testing.T) {
 	if rd.To != &s1r2 || rd.ObjectKey().Name != s1r2.Name {
 		t.Error("Pointer in diff does not point to expected value")
 	}
-	mods := StatementModifiers{AllowUnsafe: true}
+	mods := StatementModifiers{Flavor: FlavorMySQL57}
 	for _, od := range sd.ObjectDiffs() {
 		stmt, err := od.Statement(mods)
 		if stmt != "" || err != nil {
 			t.Errorf("Unexpected return from Statement: %s / %v", stmt, err)
 		}
 	}
+	mods.AllowUnsafe = true
 	mods.CompareMetadata = true
 	for n, od := range sd.ObjectDiffs() {
 		stmt, err := od.Statement(mods)
@@ -583,8 +584,24 @@ func TestSchemaDiffRoutines(t *testing.T) {
 			t.Errorf("Unexpected return from Statement: %s / %v", stmt, err)
 		}
 	}
+	mods.Flavor = FlavorMariaDB101
+	mods.AllowUnsafe = false
+	for n, od := range sd.ObjectDiffs() {
+		stmt, err := od.Statement(mods)
+		if err != nil {
+			t.Errorf("Unexpected error from Statement[%d]: %v", n, err)
+			continue
+		}
+		if n == 0 && stmt != "" {
+			t.Errorf("Expected blank statement from Statement[0], instead found %q", stmt)
+		}
+		if n == 1 && (!strings.HasPrefix(stmt, "# ") || !strings.Contains(stmt, "CREATE OR REPLACE")) {
+			t.Errorf("Unexpected statement from Statement[1]: %q", stmt)
+		}
+	}
 
-	// Confirm that procs and funcs with same name are handled properly
+	// Confirm that procs and funcs with same name are handled properly, including
+	// with respect to ignore patterns
 	s1r2 = aProc("latin1_swedish_ci", "")
 	s1.Routines = []*Routine{&s1r2} // s1 has one proc named "proc1" and no funcs
 	s2r1.Name = s2r2.Name           // s2 has one proc named "proc1" and one func also named "proc1"
@@ -875,43 +892,6 @@ func TestAlterTableStatementVirtualColValidation(t *testing.T) {
 	assertWithValidation(false)
 }
 
-func TestIgnoreTableMod(t *testing.T) {
-	from := anotherTable()
-	to := anotherTable()
-	col := &Column{
-		Name:     "something",
-		TypeInDB: "smallint(5) unsigned",
-	}
-	to.Columns = append(to.Columns, col)
-	to.CreateStatement = to.GeneratedCreateStatement(FlavorUnknown)
-	alter := NewAlterTable(&from, &to)
-	create := NewCreateTable(&from)
-	drop := NewDropTable(&from)
-	assertStatement := func(re string, tableName string, expectNonemptyStatement bool) {
-		t.Helper()
-		mods := StatementModifiers{
-			AllowUnsafe: true,
-		}
-		if re != "" {
-			mods.IgnoreTable = regexp.MustCompile(re)
-		}
-		from.Name = tableName
-		to.Name = tableName
-		if stmt, err := alter.Statement(mods); err != nil || (stmt == "") == expectNonemptyStatement {
-			t.Errorf("Unexpected result for alter: re=%s, table=%s, expectNonEmpty=%t, actual=%s, err=%s", re, tableName, expectNonemptyStatement, stmt, err)
-		}
-		if stmt, err := create.Statement(mods); err != nil || (stmt == "") == expectNonemptyStatement {
-			t.Errorf("Unexpected result for create: re=%s, table=%s, expectNonEmpty=%t, actual=%s, err=%s", re, tableName, expectNonemptyStatement, stmt, err)
-		}
-		if stmt, err := drop.Statement(mods); err != nil || (stmt == "") == expectNonemptyStatement {
-			t.Errorf("Unexpected result for drop: re=%s, table=%s, expectNonEmpty=%t, actual=%s, err=%s", re, tableName, expectNonemptyStatement, stmt, err)
-		}
-	}
-	assertStatement("", "testing", true)
-	assertStatement("^hello", "testing", true)
-	assertStatement("^test", "testing", false)
-}
-
 func TestNilObjectDiff(t *testing.T) {
 	var td *TableDiff
 	expectKey := ObjectKey{}
