@@ -312,23 +312,24 @@ func (s TengoIntegrationSuite) TestInstanceCloseAll(t *testing.T) {
 
 func (s TengoIntegrationSuite) TestInstanceFlavorVersion(t *testing.T) {
 	imageToFlavor := map[string]Flavor{
-		"mysql:5.5":    FlavorMySQL55,
-		"mysql:5.6":    FlavorMySQL56,
-		"mysql:5.7":    FlavorMySQL57,
-		"mysql:8.0":    FlavorMySQL80,
-		"percona:5.5":  FlavorPercona55,
-		"percona:5.6":  FlavorPercona56,
-		"percona:5.7":  FlavorPercona57,
-		"percona:8.0":  FlavorPercona80,
-		"mariadb:10.1": FlavorMariaDB101,
-		"mariadb:10.2": FlavorMariaDB102,
-		"mariadb:10.3": FlavorMariaDB103,
-		"mariadb:10.4": FlavorMariaDB104,
-		"mariadb:10.5": FlavorMariaDB105,
-		"mariadb:10.6": FlavorMariaDB106,
-		"mariadb:10.7": FlavorMariaDB107,
-		"mariadb:10.8": FlavorMariaDB108,
-		"mariadb:10.9": FlavorMariaDB109,
+		"mysql:5.5":     FlavorMySQL55,
+		"mysql:5.6":     FlavorMySQL56,
+		"mysql:5.7":     FlavorMySQL57,
+		"mysql:8.0":     FlavorMySQL80,
+		"percona:5.5":   FlavorPercona55,
+		"percona:5.6":   FlavorPercona56,
+		"percona:5.7":   FlavorPercona57,
+		"percona:8.0":   FlavorPercona80,
+		"mariadb:10.1":  FlavorMariaDB101,
+		"mariadb:10.2":  FlavorMariaDB102,
+		"mariadb:10.3":  FlavorMariaDB103,
+		"mariadb:10.4":  FlavorMariaDB104,
+		"mariadb:10.5":  FlavorMariaDB105,
+		"mariadb:10.6":  FlavorMariaDB106,
+		"mariadb:10.7":  FlavorMariaDB107,
+		"mariadb:10.8":  FlavorMariaDB108,
+		"mariadb:10.9":  FlavorMariaDB109,
+		"mariadb:10.10": FlavorMariaDB1010,
 	}
 
 	// Determine expected Flavor value of the Dockerized instance being tested
@@ -559,7 +560,6 @@ func (s TengoIntegrationSuite) TestInstanceCreateSchema(t *testing.T) {
 	opts := SchemaCreationOptions{
 		DefaultCharSet:   "utf8mb4",
 		DefaultCollation: "utf8mb4_unicode_ci",
-		SkipBinlog:       true,
 	}
 	_, err := s.d.CreateSchema("foobar", opts)
 	if err != nil {
@@ -631,7 +631,6 @@ func (s TengoIntegrationSuite) TestInstanceDropTablesInSchemaByRef(t *testing.T)
 	}
 	opts := BulkDropOptions{
 		MaxConcurrency: 10,
-		SkipBinlog:     true,
 		Schema:         schema,
 	}
 	if err := s.d.DropTablesInSchema("testing", opts); err != nil {
@@ -666,7 +665,7 @@ func (s TengoIntegrationSuite) TestInstanceDropTablesDeadlock(t *testing.T) {
 	// Add a FK relation, drop all tables in the schema, and then restore the
 	// test database to its previous state. Without the fix in DropTablesInSchema,
 	// this tends to hit a deadlock within just a few loop iterations.
-	opts := BulkDropOptions{MaxConcurrency: 10, SkipBinlog: true}
+	opts := BulkDropOptions{MaxConcurrency: 10}
 	for n := 0; n < 10; n++ {
 		_, err = db.Exec("ALTER TABLE testing.actor_in_film ADD CONSTRAINT actor FOREIGN KEY (actor_id) REFERENCES testing.actor (actor_id)")
 		if err != nil {
@@ -748,7 +747,6 @@ func (s TengoIntegrationSuite) TestInstanceDropRoutinesInSchema(t *testing.T) {
 	}
 	opts := BulkDropOptions{
 		MaxConcurrency: 10,
-		SkipBinlog:     true,
 	}
 	if err := s.d.DropRoutinesInSchema("testing", opts); err != nil {
 		t.Fatalf("Unexpected error from DropRoutinesInSchema: %v", err)
@@ -776,7 +774,6 @@ func (s TengoIntegrationSuite) TestInstanceDropRoutinesInSchemaByRef(t *testing.
 	}
 	opts := BulkDropOptions{
 		MaxConcurrency: 10,
-		SkipBinlog:     true,
 		Schema:         schema,
 	}
 	if err := s.d.DropRoutinesInSchema("testing", opts); err != nil {
