@@ -1090,6 +1090,12 @@ func TestTableAlterChangeCharSet(t *testing.T) {
 	from = getTableWithCharSet("utf8", "utf8_general_ci", true)
 	to = getTableWithCharSet("utf8mb3", "utf8_general_ci", true)
 	assertChangeCharSet(&from, &to, "")
+	to = getTableWithCharSet("utf8mb3", "utf8mb3_general_ci", true)
+	assertChangeCharSet(&from, &to, "")
+
+	// Confirm "utf8" and "utf8mb4" are not treated as identical
+	to = getTableWithCharSet("utf8mb4", "utf8mb4_general_ci", true)
+	assertChangeCharSet(&from, &to, "DEFAULT CHARACTER SET = utf8mb4 COLLATE = utf8mb4_general_ci")
 }
 
 func TestTableAlterChangeCreateOptions(t *testing.T) {
@@ -1190,6 +1196,40 @@ func TestTableAlterChangeComment(t *testing.T) {
 	assertChangeComment(&to, &from, "COMMENT ''")
 }
 
+func TestTableAlterTablespace(t *testing.T) {
+	getTableWithTablespace := func(tablespace string) *Table {
+		t := aTable(123)
+		t.Tablespace = tablespace
+		t.CreateStatement = t.GeneratedCreateStatement(FlavorUnknown)
+		return &t
+	}
+	assertChangeTablespace := func(a, b *Table, expectDiff bool, expectClause string) {
+		t.Helper()
+		tableAlters, supported := a.Diff(b)
+		var expectedCount int
+		if expectDiff {
+			expectedCount++
+		}
+		if len(tableAlters) != expectedCount || !supported {
+			t.Errorf("Incorrect result from Table.Diff(): %d alter clauses, supported=%t", len(tableAlters), supported)
+		} else if expectDiff {
+			if ta, ok := tableAlters[0].(ChangeTablespace); !ok {
+				t.Errorf("Incorrect type of alter returned: expected %T, found %T", ta, tableAlters[0])
+			} else if actual := ta.Clause(StatementModifiers{}); actual != expectClause {
+				t.Errorf("Incorrect ALTER TABLE clause returned: expected %q, found %q", expectClause, actual)
+			}
+		}
+	}
+
+	noTablespace := getTableWithTablespace("")
+	explicitFPT := getTableWithTablespace("innodb_file_per_table")
+	explicitSys := getTableWithTablespace("innodb_system")
+	assertChangeTablespace(noTablespace, explicitFPT, true, "TABLESPACE `innodb_file_per_table`")
+	assertChangeTablespace(explicitFPT, noTablespace, true, "") // no way to remove an explicit tablespace clause, but diff still supported
+	assertChangeTablespace(explicitFPT, explicitFPT, false, "")
+	assertChangeTablespace(explicitFPT, explicitSys, true, "TABLESPACE `innodb_system`")
+}
+
 func TestTableAlterUnsupportedTable(t *testing.T) {
 	from, to := unsupportedTable(), unsupportedTable()
 	newCol := &Column{
