@@ -91,7 +91,7 @@ func (s SkeemaIntegrationSuite) TestInitHandler(t *testing.T) {
 	// init should fail if a parent dir has an invalid .skeema file
 	fs.MakeTestDirectory(t, "hasbadoptions")
 	fs.WriteTestFile(t, "hasbadoptions/.skeema", "invalid file will not parse")
-	s.handleCommand(t, CodeFatalError, "hasbadoptions", "skeema init -h %s -P %d", s.d.Instance.Host, s.d.Instance.Port)
+	s.handleCommand(t, CodeBadConfig, "hasbadoptions", "skeema init -h %s -P %d", s.d.Instance.Host, s.d.Instance.Port)
 
 	// init should fail if the --dir specifies an existing non-directory file; or
 	// if the --dir already contains a subdir matching a schema name; or if the
@@ -858,13 +858,14 @@ func (s SkeemaIntegrationSuite) TestIgnoreOptions(t *testing.T) {
 		t.Error("Expected `skeema lint` to ignore mydb/analytics/_trending.sql, but it did not")
 	}
 
-	// push, pull, lint, format, init: invalid regexes should error. Error is
-	// CodeBadConfig except for cases of invalid ignore-schema being hit in
-	// fs.Dir.SchemaNames().
+	// push, pull, lint, format, init: invalid regexes should error with
+	// CodeBadConfig (except for push with invalid ignore-schema, which needs
+	// some further refactoring to handle configuration errors differently in
+	// some code paths)
 	s.handleCommand(t, CodeBadConfig, ".", "skeema lint --ignore-table='+'")
 	s.handleCommand(t, CodeBadConfig, ".", "skeema format --ignore-table='+'")
 	s.handleCommand(t, CodeBadConfig, ".", "skeema pull --ignore-table='+'")
-	s.handleCommand(t, CodeFatalError, ".", "skeema pull --ignore-schema='+'")
+	s.handleCommand(t, CodeBadConfig, ".", "skeema pull --ignore-schema='+'")
 	s.handleCommand(t, CodeBadConfig, ".", "skeema push --ignore-table='+'")
 	s.handleCommand(t, CodeFatalError, ".", "skeema push --ignore-schema='+'")
 	s.handleCommand(t, CodeBadConfig, ".", "skeema init --dir badre1 -h %s -P %d --ignore-schema='+'", s.d.Instance.Host, s.d.Instance.Port)
@@ -877,11 +878,11 @@ func (s SkeemaIntegrationSuite) TestDirEdgeCases(t *testing.T) {
 	// Invalid option file should break all commands
 	oldContents := fs.ReadTestFile(t, "mydb/.skeema")
 	fs.WriteTestFile(t, "mydb/.skeema", "invalid contents\n")
-	s.handleCommand(t, CodeFatalError, "mydb", "skeema pull")
-	s.handleCommand(t, CodeFatalError, "mydb", "skeema diff")
-	s.handleCommand(t, CodeFatalError, "mydb", "skeema lint")
-	s.handleCommand(t, CodeFatalError, "mydb", "skeema format")
-	s.handleCommand(t, CodeFatalError, ".", "skeema add-environment --host my.staging.db.com --dir mydb staging")
+	s.handleCommand(t, CodeBadConfig, "mydb", "skeema pull")
+	s.handleCommand(t, CodeBadConfig, "mydb", "skeema diff")
+	s.handleCommand(t, CodeBadConfig, "mydb", "skeema lint")
+	s.handleCommand(t, CodeBadConfig, "mydb", "skeema format")
+	s.handleCommand(t, CodeBadConfig, ".", "skeema add-environment --host my.staging.db.com --dir mydb staging")
 	fs.WriteTestFile(t, "mydb/.skeema", oldContents)
 
 	// Hidden directories are ignored, even if they contain a .skeema file, whether
@@ -1244,18 +1245,30 @@ END`
 		t.Error("Expected UNIX-style line-ends to be restored after `skeema pull`, but they were not")
 	}
 
-	// Modify the db representation of the routine; diff/push should work, but only
-	// with --allow-unsafe (and not with --safe-below-size)
+	// Modify the db representation of the routine. In MySQL/Percona, diff/push
+	// should work, but only with --allow-unsafe (and not with --safe-below-size).
+	// In MariaDB, --allow-unsafe is not required due to CREATE OR REPLACE support.
 	s.dbExec(t, "product", "DROP FUNCTION routine1")
 	create = strings.Replace(create, "a * b", "b * a", 1)
 	if create == origCreate {
 		t.Fatal("Test setup incorrect")
 	}
 	s.dbExec(t, "product", create)
-	s.handleCommand(t, CodeFatalError, ".", "skeema diff")
-	s.handleCommand(t, CodeDifferencesFound, ".", "skeema diff --allow-unsafe")
-	s.handleCommand(t, CodeFatalError, ".", "skeema push --safe-below-size=10000")
-	cfg = s.handleCommand(t, CodeSuccess, ".", "skeema push --allow-unsafe")
+	if s.d.Flavor().Vendor == tengo.VendorMariaDB {
+		s.handleCommand(t, CodeDifferencesFound, ".", "skeema diff")
+		s.handleCommand(t, CodeDifferencesFound, ".", "skeema diff --ignore-proc=routine1")
+		s.handleCommand(t, CodeDifferencesFound, ".", "skeema diff --ignore-func=nomatch")
+		s.handleCommand(t, CodeSuccess, ".", "skeema diff --ignore-func=routine1")
+		cfg = s.handleCommand(t, CodeSuccess, ".", "skeema push")
+	} else {
+		s.handleCommand(t, CodeFatalError, ".", "skeema diff")
+		s.handleCommand(t, CodeFatalError, ".", "skeema diff --ignore-proc=routine1")
+		s.handleCommand(t, CodeFatalError, ".", "skeema diff --ignore-func=nomatch")
+		s.handleCommand(t, CodeSuccess, ".", "skeema diff --ignore-func=routine1")
+		s.handleCommand(t, CodeDifferencesFound, ".", "skeema diff --allow-unsafe")
+		s.handleCommand(t, CodeFatalError, ".", "skeema push --safe-below-size=10000")
+		cfg = s.handleCommand(t, CodeSuccess, ".", "skeema push --allow-unsafe")
+	}
 	s.verifyFiles(t, cfg, "../golden/routines")
 	s.handleCommand(t, CodeSuccess, ".", "skeema diff")
 
@@ -1270,9 +1283,14 @@ END`
 	s.dbExec(t, "", "ALTER DATABASE product DEFAULT COLLATE = latin1_general_ci")
 	s.handleCommand(t, CodeSuccess, ".", "skeema pull")
 	s.handleCommand(t, CodeSuccess, ".", "skeema diff")
-	s.handleCommand(t, CodeFatalError, ".", "skeema diff --compare-metadata")
-	s.handleCommand(t, CodeDifferencesFound, ".", "skeema diff --compare-metadata --allow-unsafe")
-	s.handleCommand(t, CodeSuccess, ".", "skeema push --compare-metadata --allow-unsafe")
+	if s.d.Flavor().Vendor == tengo.VendorMariaDB {
+		s.handleCommand(t, CodeDifferencesFound, ".", "skeema diff --compare-metadata")
+		s.handleCommand(t, CodeSuccess, ".", "skeema push --compare-metadata")
+	} else {
+		s.handleCommand(t, CodeFatalError, ".", "skeema diff --compare-metadata")
+		s.handleCommand(t, CodeDifferencesFound, ".", "skeema diff --compare-metadata --allow-unsafe")
+		s.handleCommand(t, CodeSuccess, ".", "skeema push --compare-metadata --allow-unsafe")
+	}
 	s.handleCommand(t, CodeSuccess, ".", "skeema diff --compare-metadata")
 	s.d.CloseAll() // avoid mysql bug where ALTER DATABASE doesn't affect existing sessions
 
@@ -1351,14 +1369,37 @@ END`
 	}
 }
 
+// TestTempSchemaBinlog provides coverage for the temp-schema-binlog option.
+// Because we ordinarily create containerized test DBs with binlogging disabled
+// (even in MySQL 8 where it normally defaults to enabled), this test has to
+// create a new separate container for its logic.
+// This test is run in CI, or when SKEEMA_TEST_BINLOG env var is set to any non-
+// blank value.
 func (s SkeemaIntegrationSuite) TestTempSchemaBinlog(t *testing.T) {
-	if !s.d.Flavor().Min(tengo.FlavorMySQL80) {
-		t.Skip("Test only relevant for flavors that default to having binlog enabled")
+	if os.Getenv("SKEEMA_TEST_BINLOG") == "" && (os.Getenv("CI") == "" || os.Getenv("CI") == "0" || os.Getenv("CI") == "false") {
+		t.Skip("Skipping temp-schema-binlog testing. To run, set env var SKEEMA_TEST_BINLOG=true and/or CI=1.")
+	}
+
+	// Create an instance with log-bin enabled
+	opts := s.d.DockerizedInstanceOptions
+	opts.Name = strings.Replace(opts.Name, "skeema-test-", "skeema-test-binlog-", 1)
+	opts.CommandArgs = []string{"--log-bin", "--server-id=1"}
+	dinst, err := s.manager.GetOrCreateInstance(opts)
+	if err != nil {
+		t.Fatalf("Unable to create Dockerized instance with log-bin enabled: %v", err)
+	}
+	defer func() {
+		if err := dinst.Destroy(); err != nil {
+			t.Errorf("Unable to destroy test instance with log-bin enabled: %v", err)
+		}
+	}()
+	if _, err := dinst.SourceSQL("../setup.sql"); err != nil {
+		t.Fatalf("Unable to source setup.sql: %v", err)
 	}
 
 	getLogPos := func() string {
 		t.Helper()
-		db, err := s.d.CachedConnectionPool("", "")
+		db, err := dinst.CachedConnectionPool("", "")
 		if err != nil {
 			t.Fatalf("Unable to establish connection: %v", err)
 		}
@@ -1392,9 +1433,8 @@ func (s SkeemaIntegrationSuite) TestTempSchemaBinlog(t *testing.T) {
 	}
 
 	pos := getLogPos()
-	s.handleCommand(t, CodeSuccess, ".", "skeema init --dir mydb -h %s -P %d", s.d.Instance.Host, s.d.Instance.Port)
+	s.handleCommand(t, CodeSuccess, ".", "skeema init --dir mydb -h %s -P %d", dinst.Instance.Host, dinst.Instance.Port)
 	assertNotLogged(pos)
-	s.dbExec(t, "analytics", "ALTER TABLE pageviews DROP COLUMN domain")
 	createRoutine := `CREATE definer=root@localhost FUNCTION routine1(a int,
   b int)
 RETURNS int
