@@ -58,11 +58,13 @@ func (s *SkeemaIntegrationSuite) Setup(backend string) (err error) {
 	}
 
 	// Spin up a Dockerized database server
-	s.d, err = s.manager.GetOrCreateInstance(tengo.DockerizedInstanceOptions{
+	opts := tengo.DockerizedInstanceOptions{
 		Name:         fmt.Sprintf("skeema-test-%s", tengo.ContainerNameForImage(backend)),
 		Image:        backend,
 		RootPassword: "fakepw",
-	})
+		CommandArgs:  []string{"--skip-log-bin"}, // override MySQL 8 default of enabling binlog
+	}
+	s.d, err = s.manager.GetOrCreateInstance(opts)
 	return err
 }
 
@@ -359,6 +361,11 @@ func (s *SkeemaIntegrationSuite) compareDirLogicalSchemas(t *testing.T, a, b *fs
 				if flavor.OmitIntDisplayWidth() {
 					aText = reDisplayWidth.ReplaceAllString(aText, "$1$3$4")
 				}
+				if flavor.AlwaysShowCollate() {
+					aText = strings.Replace(aText, "DEFAULT CHARSET=latin1;", "DEFAULT CHARSET=latin1 COLLATE=latin1_swedish_ci;", 1)
+					aText = strings.Replace(aText, "DEFAULT CHARSET=latin1\n", "DEFAULT CHARSET=latin1 COLLATE=latin1_swedish_ci\n", 1)
+					aText = strings.Replace(aText, "DEFAULT CHARSET=utf8mb4;", "DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;", 1)
+				}
 				if aText != bText {
 					t.Errorf("Mismatch for %s:\n%s:\n%s\n\n%s:\n%s\n", key, aStmt.Location(), aText, bStmt.Location(), bText)
 				}
